use crate::models::HoldRole;
use crate::models::{self};
use leptos::prelude::*;

#[component]
#[tracing::instrument(skip_all)]
pub fn Problem(#[prop(into)] dim: Signal<models::WallDimensions>, #[prop(into)] problem: Signal<models::Problem>) -> impl IntoView {
    let holds = move || {
        let mut holds = vec![];
        for row in 0..dim.get().rows {
            for col in 0..dim.get().cols {
                let hold_position = models::HoldPosition { row, col };
                let role = move || problem.get().holds.get(&hold_position).copied();
                let role = Signal::derive(role);
                let hold = view! { <Hold role /> };
                holds.push(hold);
            }
        }
        holds.into_iter().collect_view()
    };

    let grid_classes = move || format!("grid grid-rows-{} grid-cols-{} gap-3", dim.get().rows, dim.get().cols);

    view! {
        <div class="grid grid-cols-[auto,1fr] gap-8">
            <div class=move || { grid_classes }>{holds}</div>
        </div>
    }
}

#[component]
#[tracing::instrument(skip_all)]
fn Hold(#[prop(into)] role: Signal<Option<HoldRole>>) -> impl IntoView {
    let class = move || {
        let role_classes = match role.get() {
            Some(HoldRole::Start) => Some("outline outline-offset-2 outline-green-500"),
            Some(HoldRole::Normal) => Some("outline outline-offset-2 outline-blue-500"),
            Some(HoldRole::Zone) => Some("outline outline-offset-2 outline-amber-500"),
            Some(HoldRole::End) => Some("outline outline-offset-2 outline-red-500"),
            None => Some("brightness-50"),
        };
        let mut s = "min-w-2 bg-sky-100 aspect-square rounded".to_string();
        if let Some(c) = role_classes {
            s.push(' ');
            s.push_str(c);
        }
        s
    };

    view! { <div class=class /> }
}
